/* ===========================================================================
Copyright (C) 2002-2005  3Dlabs Inc. Ltd.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

    Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

    Redistributions in binary form must reproduce the above
    copyright notice, this list of conditions and the following
    disclaimer in the documentation and/or other materials provided
    with the distribution.

    Neither the name of 3Dlabs Inc. Ltd. nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
=========================================================================== */

//
// The source of the shaders itself are in plain text files. Files ending in
// .vert are vertex shaders, and files ending in .frag are fragment shaders.
//

#include "pond.h"
#include <wand/magick_wand.h>
#include "obj.h"

//
// Handle of the window we're rendering to
//
static GLint window;
GLuint bumpProg, waterProg;

//
// Models to render
//
EModelType gleModel = EModelCube;

//
// Movement variables
//
float fXDiff = 0;
float fYDiff = 0;
float fZDiff = 0;
int xLastIncr = 0;
int yLastIncr = 0;
float fXInertia = -0.5;
float fYInertia = 0;
float fXInertiaOld;
float fYInertiaOld;
float fScale = 1.0;
float ftime = 0;
int xLast = -1;
int yLast = -1;
char bmModifiers;
int Rotate = 0;
float LastTime = 0;
int frames = 0;

// init light position
float lpos[3] = {0.0, 30.0, -3.0};
float aspect = 1.0;
int textureID = 0;

int winWidth = 512, winHeight = 512;
const float cameraInitPos[3] = {0.0, 10.0, 40.0};
float cameraPos[3];
float cameraRot[2] = {0.0, 0.0};
const float deltaRot[2] = {2, 2};
float cameraDir[3] = {0.0, 0.0, 1.0};
const float radian = 3.14f/180;
float cameraHeight = 5;
float waterHeight = -1.0;
int rock1, sky, wallObj[2], floorObj[3], smallRocks;
float waterOffset[2] = {0.0, 0.0};
float waterX = 0.001, waterY = 0.001;

GLuint rockBump, waterDuDv, waterBump;
GLuint reflectionTex, refracTex;
GLuint wallBump[2];
GLuint floorBump[3];

int texSize = 512;

#define HEIGHT_INCREMENT 0.2;

//
// Rotation defines
//
#define INERTIA_THRESHOLD       1.0f
#define INERTIA_FACTOR          0.5f
#define SCALE_FACTOR            0.01f
#define SCALE_INCREMENT         0.1f
#define TIMER_FREQUENCY_MILLIS  20

/*public*/
int printOglError(char *file, int line)
{
    //
    // Returns 1 if an OpenGL error occurred, 0 otherwise.
    //
    GLenum glErr;
    int    retCode = 0;

    glErr = glGetError();
    while (glErr != GL_NO_ERROR)
    {
        printf("glError in file %s @ line %d: %s\n", file, line, gluErrorString(glErr));
        retCode = 1;
        glErr = glGetError();
    }
    return retCode;
}


/***************************************************************************/
/* Parse GL_VERSION and return the major and minor numbers in the supplied
 * integers.
 * If it fails for any reason, major and minor will be set to 0.
 * Assumes a valid OpenGL context.
*/

void getGlVersion( int *major, int *minor )
{
    const char* verstr = (const char*)glGetString( GL_VERSION );
    if( (verstr == NULL) || (sscanf( verstr, "%d.%d", major, minor ) != 2) )
    {
        *major = *minor = 0;
        fprintf( stderr, "Invalid GL_VERSION format!!!\n" );
    }
}


static void NextClearColor(void)
{
    static int color = 0;

    switch( color++ )
    {
        case 0:  glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
             break;
        case 1:  glClearColor(0.2f, 0.2f, 0.3f, 1.0f);
             break;
        default: glClearColor(0.7f, 0.7f, 0.7f, 1.0f);
             color = 0;
             break;
    }
}



/******************************************************************************

 GLUT glue

******************************************************************************/

void reshape(int wid, int ht)
{
	winWidth = wid;
	winHeight = ht;
	aspect = (float) wid / (float) ht;

	glViewport(0, 0, wid, ht);

 	// set perspective projection
	glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
	gluPerspective(60.0, aspect, 0.1, 1024);
   glMatrixMode(GL_MODELVIEW);
	
   glUseProgram(waterProg);
	glUniform2f(getUniLoc(waterProg, "winSize"), winWidth, winHeight);
	if (winWidth > 1024 && winHeight > 1024) {
		texSize = 1024;
	}	
	else {
		texSize = 512;
	}	
}

// drawing water surface
void drawWater()
{
	// using two textures and DuDv, normal map
	// to draw water using water shader
	glUseProgram(waterProg);
	glActiveTexture(GL_TEXTURE3);
	glBindTexture(GL_TEXTURE_2D, waterBump);
	glActiveTexture(GL_TEXTURE2);
	glBindTexture(GL_TEXTURE_2D, waterDuDv);
	glActiveTexture(GL_TEXTURE1);	 	 
	glBindTexture(GL_TEXTURE_2D, refracTex);
	glActiveTexture(GL_TEXTURE0);	 	 
	glBindTexture(GL_TEXTURE_2D, reflectionTex);

	/* disable back face culling for water surface    */
	/* to see refracted scene from the bottom of pond */
	glDisable(GL_CULL_FACE);
	glPushMatrix();
		glTranslatef(0.0f, waterHeight, 0.0f);
		glScalef(40.0f, 1.0f, 40.0f); 
		glBegin( GL_POLYGON );
			glTexCoord2f( 0.0f, 0.0f );
			glVertex3f( -1.0f, 0.0f, 1.0f);

			glTexCoord2f( 1.0f, 0.0f );
			glVertex3f(  1.0f, 0.0f, 1.0f);

			glTexCoord2f( 1.0f, 1.0f );        
			glVertex3f(  1.0f,  0.0f, -1.0f);

			glTexCoord2f( 0.0f, 1.0f );     
			glVertex3f( -1.0f,  0.0f, -1.0f);
		glEnd();    
	glPopMatrix();
	glEnable(GL_CULL_FACE);
}

// draw whole scene with three passes
// pass 0 - create reflection texture
// pass 1 - create refraction texture
// pass 2 - draw real scene and water
void drawScene(int pass)
{
	glPushMatrix();
	
	// set viewport fitting to texture size 512x512 or 1024x1024
	// for pass 0 and 1
	if (pass < 2)
		glViewport(0, 0, texSize, texSize);
	else 		
		glViewport(0, 0, winWidth, winHeight); // back to window size viewport for pass 2

	// no clipping for creating refraction texture
	if (pass == 1)
		glDisable(GL_CLIP_PLANE0);
	else
		glEnable(GL_CLIP_PLANE0);
	
	// for reflection			
	if (pass == 0) {
		// faces are flipped when flip scene upside down
		// so culling front face
		glCullFace(GL_FRONT);
		
		// flip scene upside down and translate back
		glTranslatef(0.0f, waterHeight, 0.0f);
		glScalef(1.0f, -1.0f, 1.0f);

		// clip downside
		double plane[4] = {0.0, 1.0, 0.0, 0.0};
		glClipPlane(GL_CLIP_PLANE0, plane);
		
		// translate water surface to surface y=0
		glTranslatef(0.0f, -waterHeight, 0.0f);
		
		// turn off fog for reflection texture
	   glUseProgram(bumpProg);
		glUniform1i(getUniLoc(bumpProg, "fogOn"), 0);
	}
	else {
		glCullFace(GL_BACK);
		// turn on fog for refraction and real scene
		glUseProgram(bumpProg);
		glUniform1i(getUniLoc(bumpProg, "fogOn"), 1);
	}

	if (pass == 2) {
		// clip downside when camera.y > water height
		double plane[4] = {0.0, 1.0, 0.0, 0.001};
		// clip upside when camera.y < water height
		if (cameraPos[1] < waterHeight) {
			plane[1] = -1.0;
			plane[3] = 0.001;
		}

		// translate back
		glTranslatef(0.0f, waterHeight, 0.0f);
		// clip
		glClipPlane(GL_CLIP_PLANE0, plane);
		// translate water surface to surface y=0
		glTranslatef(0.0f, -waterHeight, 0.0f);
	}
		
	// draw sky using fixed functionality
	glUseProgram(0);
   obj_draw_file(sky);

	// using bump mapping shader for all other models
   glUseProgram(bumpProg);

	// textuer0 - color map, texture1 - bump map
	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, wallBump[0]);
	glActiveTexture(GL_TEXTURE0);	 	 
	obj_draw_file(wallObj[0]);

	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, wallBump[1]);
	glActiveTexture(GL_TEXTURE0);	 	 
	obj_draw_file(wallObj[1]);

	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, floorBump[0]);
	glActiveTexture(GL_TEXTURE0);	 	 
	obj_draw_file(floorObj[0]);

	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, floorBump[1]);
	glActiveTexture(GL_TEXTURE0);	 	 
	obj_draw_file(floorObj[1]);

	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, floorBump[2]);
	glActiveTexture(GL_TEXTURE0);	 	 
	obj_draw_file(floorObj[2]);	

	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, rockBump);
	glActiveTexture(GL_TEXTURE0);	 	 

	obj_draw_file(rock1);
	obj_draw_file(smallRocks);

	// draw water surface
	if (pass == 2) {
		drawWater();		
	}
	
	glPopMatrix();
}

void display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	//set camera for objects
   gluLookAt(cameraPos[0], cameraPos[1], cameraPos[2], 
			cameraPos[0]+cameraDir[0], cameraPos[1]+cameraDir[1], cameraPos[2]+cameraDir[2], 
			0.0f, 1.0f, 0.0f);

	//create reflection texture - pass 0
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	drawScene(0);
	glBindTexture(GL_TEXTURE_2D, reflectionTex);
   glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 0, 0, texSize, texSize, 0);

	//create refraction texture - pass 1
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	drawScene(1);
	glBindTexture(GL_TEXTURE_2D, refracTex);
   glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 0, 0, texSize, texSize, 0);

	//draw scene with water	- pass 2
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	drawScene(2);

	glFlush();
	glutSwapBuffers();
}


static
void play(void)
{
	// set camera direction
	cameraDir[0] = sin(cameraRot[0]*radian);
	cameraDir[1] = tan(cameraRot[1]*radian);
	cameraDir[2] = -cos(cameraRot[0]*radian);

	int thisTime = glutGet(GLUT_ELAPSED_TIME);
	frames++;
	if (frames >= 100) {
   	float dT =  thisTime - LastTime;
		float fps = frames/dT*1000.0;
		printf("fps = %f\n", fps);
		frames = 0;
		LastTime = thisTime;
	}
	
	glutPostRedisplay();
}


static
void key(unsigned char keyPressed, int x, int y)
{
    switch(keyPressed)
    {
        case 'b':
            NextClearColor();
            break;
        case 'q':
        case 27:
            exit(0);
            break;
        case 't':
            textureID = (textureID + 1) % 3;
            break;
        case ' ':
            Rotate = !Rotate;

            if (!Rotate)
            {
                fXInertiaOld = fXInertia;
                fYInertiaOld = fYInertia;
            }
            else
            {
                fXInertia = fXInertiaOld;
                fYInertia = fYInertiaOld;

                // To prevent confusion, force some rotation
                if ((fXInertia == 0.0) && (fYInertia == 0.0))
                    fXInertia = -0.5;
            }
            break;
        case '+':
            cameraPos[1] += HEIGHT_INCREMENT;
			   glUseProgram(bumpProg);
				glUniform3fv(getUniLoc(bumpProg, "eyePos"), 1, cameraPos);
            break;
        case '-':
		      cameraPos[1] -= HEIGHT_INCREMENT;
	  		   glUseProgram(bumpProg);
				glUniform3fv(getUniLoc(bumpProg, "eyePos"), 1, cameraPos);
            break;
        default:
            fprintf(stderr, "\nKeyboard commands:\n\n"
            "q, <esc> - Quit\n"
            "? - Help\n"
            "<home>   - reset camera position and direction\n"
            "<+>, <-> - camera up/down\n"
            "<arrow keys> - rotate camera\n"
				"<pageUP/DOWN> - move camera forward/backward\n");
            break;
    }
}


static
void timer(int value)
{
	// water animation by increasing DuDv/Normal map coordinate offsets
	waterOffset[0] += waterX;
	waterOffset[1] += waterY;
	
   glUseProgram(waterProg);
	glUniform2fv(getUniLoc(waterProg, "coordOffset"), 1, waterOffset);
		 
   /* Callback */
   glutTimerFunc(TIMER_FREQUENCY_MILLIS , timer, 0);
}


static
void mouse(int button, int state, int x, int y)
{
   bmModifiers = glutGetModifiers();

   if (button == GLUT_LEFT_BUTTON)
   {
      if (state == GLUT_UP)
      {
         xLast = -1;
         yLast = -1;
			/*
         if (xLastIncr > INERTIA_THRESHOLD)
            fXInertia = (xLastIncr - INERTIA_THRESHOLD)*INERTIA_FACTOR;

         if (-xLastIncr > INERTIA_THRESHOLD) 
            fXInertia = (xLastIncr + INERTIA_THRESHOLD)*INERTIA_FACTOR;

         if (yLastIncr > INERTIA_THRESHOLD) 
            fYInertia = (yLastIncr - INERTIA_THRESHOLD)*INERTIA_FACTOR;

         if (-yLastIncr > INERTIA_THRESHOLD) 
            fYInertia = (yLastIncr + INERTIA_THRESHOLD)*INERTIA_FACTOR;
				*/
      }
      else
      {
			/*
         fXInertia = 0;
         fYInertia = 0;
			*/
      }
      xLastIncr = 0;
      yLastIncr = 0;
   }
}


static
void motion(int x, int y)
{

   if ((xLast != -1) || (yLast != -1))
   {
      xLastIncr = x - xLast;
      yLastIncr = y - yLast;
      if (bmModifiers & GLUT_ACTIVE_CTRL)
      {
         if (xLast != -1) {
            //cameraPos[0] -= 0.5*xLastIncr;
         }
      }
      else
      {
         if (xLast != -1)
         {
            cameraPos[0] -= 0.5*xLastIncr;
            cameraPos[1] -= 0.5*yLastIncr;
		   	glUseProgram(bumpProg);
				glUniform3fv(getUniLoc(bumpProg, "eyePos"), 1, cameraPos);
         }
      }
   }

   xLast = x;
   yLast = y;
}

static
void special(int key, int x, int y)
{
   switch (key) {
		case GLUT_KEY_HOME:
   		fXDiff    = 0;
   		fYDiff    = 0;
   		fZDiff    = 0;
   		xLastIncr = 0;
   		yLastIncr = 0;
   		fXInertia = -0.5;
   		fYInertia = 0;
   		fScale    = 1.0;
			memcpy(cameraPos, cameraInitPos, sizeof(float)*3);
		   glUseProgram(bumpProg);
			glUniform3fv(getUniLoc(bumpProg, "eyePos"), 1, cameraPos);
			cameraRot[0] = 0.0;
			cameraRot[1] = 0.0;
		break;
		 
		case GLUT_KEY_LEFT:
			cameraRot[0] = cameraRot[0] - deltaRot[0];
		break;
		case GLUT_KEY_RIGHT:
			cameraRot[0] = cameraRot[0] + deltaRot[1];
		break;
		case GLUT_KEY_UP:
			cameraRot[1] = cameraRot[1] + deltaRot[1];
		break;
		case GLUT_KEY_DOWN:
			cameraRot[1] = cameraRot[1] - deltaRot[1];
		break;
		case GLUT_KEY_PAGE_UP:
			cameraPos[0] = cameraPos[0] + cameraDir[0];
			cameraPos[2] = cameraPos[2] + cameraDir[2];
		   glUseProgram(bumpProg);
			glUniform3fv(getUniLoc(bumpProg, "eyePos"), 1, cameraPos);
		break;
		case GLUT_KEY_PAGE_DOWN:
			cameraPos[0] = cameraPos[0] - cameraDir[0];
			cameraPos[2] = cameraPos[2] - cameraDir[2];
		   glUseProgram(bumpProg);
			glUniform3fv(getUniLoc(bumpProg, "eyePos"), 1, cameraPos);
		break;

    }
}

// read an image using ImageMagick and load it to texture
GLuint loadTexture(char *fileName)
{
	MagickBooleanType status;  
	MagickWand *magick_wand;
	
	magick_wand=NewMagickWand();  
	status = MagickReadImage(magick_wand, fileName);

	if (status == MagickFalse) {
		printf("Fail to read image %s\n", fileName);
		return 1;
	}
		
	int width = MagickGetImageWidth(magick_wand);
	int height = MagickGetImageHeight(magick_wand);

	unsigned char *pixels = (unsigned char*)malloc(width*height*3);

	// convert row order
	int i;
	for (i=0; i<height; i++)
		MagickGetImagePixels(magick_wand, 0, i, width, 1, "RGB", CharPixel, 
			&pixels[(height-i-1)*width*3]);
		
	DestroyMagickWand(magick_wand);
	
	GLuint texID = 0;  
	glGenTextures(1, &texID);
	glBindTexture(GL_TEXTURE_2D, texID);
	
   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGB,
        GL_UNSIGNED_BYTE, (const GLvoid *) pixels);

	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	
	return texID;
}

// initialize texture
GLuint genTexture()
{
	GLuint texID = 0;  
	glGenTextures(1, &texID);
	glBindTexture(GL_TEXTURE_2D, texID);
	
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	
	return texID;
}

/******************************************************************************

 Main

******************************************************************************/

int main( int argc, char **argv )
{
    int gl_major, gl_minor;
    GLchar *VertexShaderSource, *FragmentShaderSource;

    glutInit( &argc, argv );
    glutInitDisplayMode( GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
    glutInitWindowSize(512, 512);
    window = glutCreateWindow( "CS594 - Proj 2");

    glutIdleFunc(play);
    glutDisplayFunc(display);
    glutKeyboardFunc(key);
    glutReshapeFunc(reshape);
    glutMotionFunc(motion);
    glutMouseFunc(mouse);
    glutSpecialFunc(special);
    glutTimerFunc(TIMER_FREQUENCY_MILLIS , timer, 0);

    // Initialize the "OpenGL Extension Wrangler" library
    glewInit();

    // Make sure that OpenGL 2.0 is supported by the driver
    getGlVersion(&gl_major, &gl_minor);
    printf("GL_VERSION major=%d minor=%d\n", gl_major, gl_minor);

    if (gl_major < 2)
    {
        printf("GL_VERSION major=%d minor=%d\n", gl_major, gl_minor);
        printf("Support for OpenGL 2.0 is required for this demo...exiting\n");
        exit(1);
    }

    glDepthFunc(GL_LESS);
    glEnable(GL_DEPTH_TEST);
	 glEnable(GL_CULL_FACE);  // enable back face culling
	 glEnable(GL_TEXTURE_2D); // enable 2D texture
	     
	 NextClearColor();

    key('?', 0, 0);     // display help

	memcpy(cameraPos, cameraInitPos, sizeof(float)*3);
	
	// load shaders
	
	// bump-map shader
		readShaderSource("bump", &VertexShaderSource, &FragmentShaderSource);
	bumpProg = installShaders(VertexShaderSource, FragmentShaderSource); // Program handle

   if (!bumpProg)
      return 0;

   glUseProgram(bumpProg);
	glUniform1i(getUniLoc(bumpProg, "colorMap"), 0);
	glUniform1i(getUniLoc(bumpProg, "normalMap"), 1);
	glUniform1f(getUniLoc(bumpProg, "waterHeight"), waterHeight);
	glUniform3fv(getUniLoc(bumpProg, "eyePos"), 1, cameraPos);
	glUniform3fv(getUniLoc(bumpProg, "lightPosition"), 1, lpos);
	glUniform1i(getUniLoc(bumpProg, "fogOn"), 0);

	// water shader
	readShaderSource("water", &VertexShaderSource, &FragmentShaderSource);
	waterProg = installShaders(VertexShaderSource, FragmentShaderSource); // Program handle

   if (!waterProg)
      return 0;

   glUseProgram(waterProg);
	glUniform1i(getUniLoc(waterProg, "reflectionMap"), 0);
	glUniform1i(getUniLoc(waterProg, "refractionMap"), 1);
	glUniform1i(getUniLoc(waterProg, "dudvMap"), 2);
	glUniform1i(getUniLoc(waterProg, "normalMap"), 3);
	glUniform2f(getUniLoc(waterProg, "winSize"), winWidth, winHeight);
	glUniform2fv(getUniLoc(waterProg, "coordOffset"), 1, waterOffset);
	glUniform3fv(getUniLoc(waterProg, "lightPosition"), 1, lpos);

	// load normal maps and DuDv maps
	rockBump = loadTexture("rock1DOT3.tga");
	waterDuDv = loadTexture("water4DUDV.tga");
	waterBump = loadTexture("water4DOT3.tga");
	wallBump[0] = loadTexture("frontWallDOT3.tga");
	wallBump[1] = loadTexture("sideWallDOT3.tga");
	floorBump[0] = loadTexture("floorDOT3.tga");
	floorBump[1] = loadTexture("pond1DOT3.tga");
	floorBump[2] = loadTexture("pond4DOT3.tga");
	
	// init textures for 
	reflectionTex = genTexture();
	refracTex = genTexture();

	// read obj models
	rock1 = obj_add_file("rock1.obj");
	smallRocks = obj_add_file("smallRock.obj");
	sky = obj_add_file("sky.obj");
	wallObj[0]  = obj_add_file("wall-0.obj");
	wallObj[1]  = obj_add_file("wall-1.obj");
	floorObj[0] = obj_add_file("floor-0.obj");
	floorObj[1] = obj_add_file("floor-1.obj");
	floorObj[2] = obj_add_file("floor-2.obj");	
	
	glutMainLoop();
    
	return 0;
}
